/* libaudit.h -- 
 * Copyright 2004, 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Authors:
 *	Steve Grubb <sgrubb@redhat.com>
 *	Rickard E. (Rik) Faith <faith@redhat.com>
 */
#ifndef _LIBAUDIT_H_
#define _LIBAUDIT_H_

#include <asm/types.h>
#include <stdint.h>
#include <sys/socket.h>
#include <linux/netlink.h>
#include <linux/audit.h>
#include <stdarg.h>
#include <syslog.h>
#include <linux/fs.h>	/* For watch */

/* TODO: Add this as a kernel patch to audit.h */
#define AUDIT_FIRST_USER_MSG    1100    /* First user space message */
#define AUDIT_LAST_USER_MSG     1199    /* Last user space message */
#define AUDIT_USER_AUTH         1100    /* User space authentication */
#define AUDIT_USER_ACCT         1101    /* User space acct change */
#define AUDIT_USER_MGMT         1102    /* User space acct management */
#define AUDIT_CRED_ACQ          1103    /* User space credential acquired */
#define AUDIT_CRED_DISP         1104    /* User space credential disposed */
#define AUDIT_USER_START        1105    /* User space session start */
#define AUDIT_USER_END          1106    /* User space session end */
#define AUDIT_USER_AVC          1107    /* User space avc message */
#define AUDIT_USER_CHAUTHTOK	1108	/* User space acct attr changed */
#define AUDIT_USER_ERR		1109	/* User space acct state err */
#define AUDIT_CRED_REFR         1110    /* User space credential refreshed */

#define AUDIT_DAEMON_START      1200    /* Daemon startup record */
#define AUDIT_DAEMON_END        1201    /* Daemon normal stop record */
#define AUDIT_DAEMON_ABORT      1202    /* Daemon error stop record */
#define AUDIT_DAEMON_CONFIG     1203    /* Daemon config change */

#define AUDIT_FIRST_EVENT	1300
#define AUDIT_LAST_EVENT	1399
#define AUDIT_SYSCALL           1300    /* Syscall event */
#define AUDIT_FS_WATCH          1301    /* Filesystem watch event */
#define AUDIT_PATH              1302    /* Filname path information */
#define AUDIT_IPC               1303    /* IPC record */
#define AUDIT_SOCKETCALL        1304    /* sys_socketcall arguments */
#define AUDIT_CONFIG_CHANGE     1305    /* Audit system configuration change */
#define AUDIT_SOCKADDR          1306    /* sockaddr copied as syscall arg */

#define AUDIT_FIRST_SELINUX	1400
#define AUDIT_LAST_SELINUX	1499
#define AUDIT_AVC               1400    /* SE Linux avc denial or grant */
#define AUDIT_SELINUX_ERR       1401    /* Internal SE Linux Errors */
#define AUDIT_AVC_PATH		1402    /* dentry, vfsmount pair from avc */


/* From the arch patch */
#ifndef AUDIT_ARCH
#include <elf.h>
#define AUDIT_ARCH		11
#define __AUDIT_ARCH_64BIT     0x80000000
#define __AUDIT_ARCH_LE        0x40000000
#define AUDIT_ARCH_ALPHA       (EM_ALPHA|__AUDIT_ARCH_64BIT|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_ARM         (EM_ARM|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_ARMEB       (EM_ARM)
#define AUDIT_ARCH_CRIS        (EM_CRIS|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_FRV         (EM_FRV)
#define AUDIT_ARCH_H8300       (EM_H8_300)
#define AUDIT_ARCH_I386        (EM_386|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_IA64        (EM_IA_64|__AUDIT_ARCH_64BIT|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_M32R        (EM_M32R)
#define AUDIT_ARCH_M68K        (EM_68K)
#define AUDIT_ARCH_MIPS        (EM_MIPS)
#define AUDIT_ARCH_MIPSEL      (EM_MIPS|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_MIPS64      (EM_MIPS|__AUDIT_ARCH_64BIT)
#define AUDIT_ARCH_MIPSEL64    (EM_MIPS|__AUDIT_ARCH_64BIT|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_PARISC      (EM_PARISC)
#define AUDIT_ARCH_PARISC64    (EM_PARISC|__AUDIT_ARCH_64BIT)
#define AUDIT_ARCH_PPC         (EM_PPC)
#define AUDIT_ARCH_PPC64       (EM_PPC64|__AUDIT_ARCH_64BIT)
#define AUDIT_ARCH_S390        (EM_S390)
#define AUDIT_ARCH_S390X       (EM_S390|__AUDIT_ARCH_64BIT)
#define AUDIT_ARCH_SH          (EM_SH)
#define AUDIT_ARCH_SHEL        (EM_SH|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_SH64        (EM_SH|__AUDIT_ARCH_64BIT)
#define AUDIT_ARCH_SHEL64      (EM_SH|__AUDIT_ARCH_64BIT|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_SPARC       (EM_SPARC)
#define AUDIT_ARCH_SPARC64     (EM_SPARC64|__AUDIT_ARCH_64BIT)
#define AUDIT_ARCH_V850        (EM_V850|__AUDIT_ARCH_LE)
#define AUDIT_ARCH_X86_64      (EM_X86_64|__AUDIT_ARCH_64BIT|__AUDIT_ARCH_LE)

#endif

/* This is the machine type list */
typedef enum {
	MACH_X86=0,
	MACH_86_64,
	MACH_IA64,
	MACH_PPC64,
	MACH_PPC,
	MACH_S390X,
	MACH_S390
} machine_t;

/* From the watch patch */
#ifndef AUDIT_WATCH_INS
#define AUDIT_WATCH_INS		1007	/* Insert file/dir watch entry */
#define AUDIT_WATCH_REM		1008	/* Remove file/dir watch entry */
#define AUDIT_WATCH_LIST	1009

/* 32 byte max key size */
#define AUDIT_FILTERKEY_MAX    32

struct audit_watch {
	uint32_t dev_major;
	uint32_t dev_minor;
	uint32_t perms;
	uint32_t valid;
	uint32_t namelen;
	uint32_t fklen;
	char     *name;
	char     *filterkey;
};

/* Structure to transport watch data to and from the kernel */
struct watch_transport {
	uint32_t dev_major;
	uint32_t dev_minor;
	uint32_t perms;
	uint32_t valid;
	uint32_t pathlen;
	uint32_t fklen;
	char buf[0];
};

#endif

#ifndef AUDIT_SIGNAL_INFO
#define AUDIT_SIGNAL_INFO         1010 /* Information about sender of signal*/

struct audit_sig_info {
        uid_t           uid;
        pid_t           pid;
};

#endif

/* defines for audit subsystem */
//struct audit_message {
//        struct nlmsghdr nlh;
//        char            data[1200]; // FIXME: bump this up to 4096*2+11+256
//};

struct audit_reply {
	struct audit_message     msg;
	int                      type;
	int                      len;
	struct nlmsghdr         *nlh;
	struct audit_status     *status;
	struct audit_rule       *rule;
	struct audit_login      *login;
	const char              *message;
	struct nlmsgerr         *error;
	struct watch_transport  *watch;
	struct audit_sig_info   *signal_info;
};

struct auditd_reply_list {
	struct audit_reply reply;
	struct auditd_reply_list *next;
};

/* Messages */
typedef enum { MSG_STDERR, MSG_SYSLOG, MSG_QUIET } message_t;
typedef enum { DBG_NO, DBG_YES } debug_message_t;
typedef enum { WAIT_NO, WAIT_YES } rep_wait_t;
void set_aumessage_mode(message_t mode, debug_message_t debug);
void audit_msg(int priority, const char *fmt, ...)
#ifdef __GNUC__
	__attribute__ ((format (printf, 2, 3)));
#else
	;
#endif

/* General */
typedef enum { GET_REPLY_BLOCKING=0, GET_REPLY_NONBLOCKING } reply_t;
extern int  audit_open(void);
extern void audit_close(int fd);
extern int  audit_get_reply(int fd, struct audit_reply *rep, reply_t block, 
		int peek);
extern int audit_send(int fd, int type, const void *data, unsigned int size);
extern uid_t audit_getloginuid(void);
extern int audit_detect_machine(void);

/* Translation functions */
extern int        audit_name_to_field(const char *field);
extern const char *audit_field_to_name(int field);
extern int        audit_name_to_syscall(const char *sc, int machine);
extern const char *audit_syscall_to_name(int sc, int machine);
extern int        audit_name_to_flag(const char *flag);
extern const char *audit_flag_to_name(int flag);
extern int        audit_name_to_action(const char *action);
extern const char *audit_action_to_name(int action);
extern int        audit_name_to_msg_type(const char *msg_type);
extern const char *audit_msg_type_to_name(int msg_type);
extern int        audit_name_to_machine(const char *machine);
extern const char *audit_machine_to_name(int machine);
extern unsigned int audit_machine_to_elf(int machine);
extern int          audit_elf_to_machine(unsigned int elf);

/* AUDIT_GET */
extern int audit_request_status(int fd);
extern int audit_is_enabled(int fd);

/* AUDIT_SET */
extern int  audit_set_enabled(int fd, int enabled);
extern int  audit_set_failure(int fd, int failure);
extern int  audit_set_pid(int fd, int pid, rep_wait_t wmode);
extern int  audit_set_rate_limit(int fd, int limit);
extern int  audit_set_backlog_limit(int fd, int limit);

/* AUDIT_LIST */
extern int  audit_request_rules_list(int fd);
extern int  audit_request_watch_list(int fd);

/* SIGNAL_INFO */
extern int audit_request_signal_info(int fd);

/* AUDIT_WATCH */
extern int audit_insert_watch(int fd, struct audit_watch *req);
extern int audit_remove_watch(int fd, struct audit_watch *req);

/* AUDIT_ADD */
extern int  audit_add_rule(int fd, struct audit_rule *rule,
                           int flags, int action);

/* AUDIT_DEL */
extern int  audit_delete_rule(int fd, struct audit_rule *rule,
                              int flags, int action);

/* AUDIT_USER */
//deprecated - do not use!
extern int audit_vlog(int fd, const char *fmt, va_list ap); //+dbus
extern int audit_send_message_if_enabled(int fd, const char *message); //+pam

// New USER message interfaces
extern int audit_vlog_if_enabled(int fd, int type, const char *fmt, 
				 va_list ap);//shadow
extern int audit_log(int fd, int type, const char *fmt, ...);
extern int audit_log_if_enabled(int fd, int type, const char *fmt, ...);//passwd
extern int audit_send_message(int fd, int type, const char *message);
extern int audit_log_avc(int fd, int type, const char *fmt, va_list ap);//dbus
extern int audit_send_user_message(int fd, int type, const char *message);//pam

/* Rule-building helper functions */
extern struct audit_rule *audit_rule_alloc(void);
extern int  audit_rule_syscall(struct audit_rule *rule, int scall);
extern int  audit_rule_syscallbyname(struct audit_rule *rule,
                                     const char *scall);
extern int  audit_rule_field(struct audit_rule *rule, int field,
                             unsigned int value);
extern int  audit_rule_fieldpair(struct audit_rule *rule, const char *pair);
extern void audit_rule_free(struct audit_rule *rule);

#endif

