/*
* ausearch-llist.h - Header file for ausearch-llist.c
* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
* All Rights Reserved.
*
* This software may be freely redistributed and/or modified under the
* terms of the GNU General Public License as published by the Free
* Software Foundation; either version 2, or (at your option) any
* later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING. If not, write to the
* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Authors:
*   Steve Grubb <sgrubb@redhat.com>
*/

#ifndef LLIST_HEADER
#define LLIST_HEADER

#include "config.h"
#include <sys/types.h>

typedef struct
{
        time_t sec;		// Event seconds
        unsigned int milli;	// millisecond of the timestamp
        unsigned long serial;	// Serial number of the event
} event;

typedef struct
{
//  int type;             // message type (KERNEL, USER, LOGIN, etc)
  pid_t pid;            // process ID
  uid_t uid;            // user ID
  uid_t euid;           // effective user ID
  uid_t loginuid;       // login user ID
  gid_t gid;            // group ID
  gid_t egid;           // effective group ID
  int success;          // success flag
  char *hostname;       // hostname
  char *filename;       // filename
  char *terminal;       // terminal
  char *exe;            // executable
  int syscall;          // syscall
} search_items;

/* This is the node of the linked list. message & item are the only elements
 * at this time. Any data elements that are per item goes here. */
typedef struct _lnode{
  char *message;	// The whole unparsed message
  int type;             // message type (KERNEL, USER, LOGIN, etc)
  unsigned int item;	// Which item of the same event
  struct _lnode* next;	// Next node pointer
} lnode;

/* This is the linked list head. Only data elements that are 1 per
 * event goes here. */
typedef struct {
  lnode *head;		// List head
  lnode *cur;		// Pointer to current node
  unsigned int cnt;	// How many items in this list

			// Data we add as 1 per event
  event e;		// event - time & serial number
  search_items s;	// items in master rec that are searchable
} llist;

void list_create(llist *l);
void list_first(llist *l);
void list_last(llist *l);
lnode *list_next(llist *l);
lnode *list_prev(llist *l);
lnode *list_get_cur(llist *l);
void list_set_cur(llist *l, lnode *n);
const char *list_message(llist *l);
void list_append(llist *l, lnode *node);

/* Given a numeric index, find that record. */
int list_find_item(llist *l, unsigned int i);

void list_clear(llist* l);

#endif

