/*
* ausearch-report.c - Minimal linked list library
* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
* All Rights Reserved. 
*
* This software may be freely redistributed and/or modified under the
* terms of the GNU General Public License as published by the Free
* Software Foundation; either version 2, or (at your option) any
* later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING. If not, write to the
* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Authors:
*   Steve Grubb <sgrubb@redhat.com>
*/

#include "config.h"
#include <stdio.h>
#include <time.h>
#include "libaudit.h"
#include "ausearch-llist.h"

/*
 * This function will take the linked list and format it for output.
 * The output order is lifo for everything except DAEMON records. DAEMON
 * records don't have serial number to separate them from one another.
 */
int output_record(llist *l)
{
	const lnode *n;

list_last(l);
	if (l->cur->type >= AUDIT_DAEMON_START &&
					l->cur->type < AUDIT_SYSCALL) {
		list_first(l);
		n = list_get_cur(l);
		if (n) {
			printf("----\ntime->%s", ctime(&l->e.sec));
			do {
				printf("%s\n", n->message);
			} while ((n=list_next(l)));
		} else {
			printf("Error - no elements in record.");
			return 1;
		}
	} else {
		list_last(l);
		n = list_get_cur(l);
		if (n) {
			printf("----\ntime->%s", ctime(&l->e.sec));
			do {
				printf("%s\n", n->message);
			} while ((n=list_prev(l)));
		} else {
			printf("Error - no elements in record.");
			return 1;
		}
	}
	return 0;
}

